'FUNCTION: Initializing EIP communication 
'@param OR_init_cfgID Tool configuration ID, see the possible configurations in the table below
'@param OR_init_fbCh Fieldbus channel: 1-4
'
'cfgID		Primary 	Secondary
'-------------------------------------------------------------------
'101	-	RG2FT		-
'102	-	RGx		-
'103	-	VG10		-
'104	-	Gecko		-
'105	HEX	-		-
'106	HEX	RGx		-
'107	HEX	VG10		-
'108	HEX	Gecko		-
'109	-	RGx		VG10
'110	-	RGx		Gecko
'111	-	RGx		RGx
'112	-	VG10		RGx
'113	-	VG10		Gecko
'114	-	VG10		VG10

'OR_init_fbCH -> 1 if Channel 1 is used
'OR_init_fbCH -> 2 if Channel 2 is used
'OR_init_fbCH -> 3 if Channel 3 is used
'OR_init_fbCH -> 4 if Channel 4 is used

UsrProc OR_init(OR_init_cfgID AS INTEGER, OR_init_fbCh AS INTEGER )
	'Select the correct bit offsert according to tool configuration
	SWITCH OR_init_cfgID
		CASE 101
		CASE 102
		CASE 103
		CASE 104
		CASE 105
			OR_primR_bitOffset = 0
			OR_primW_bitOffset = 0
			OR_secR_bitOffset = 0
			OR_secW_bitOffset = 0
		BREAK 
	
		CASE 106
		CASE 107
		CASE 108
			OR_primR_bitOffset = 192
			OR_primW_bitOffset = 128
			OR_secR_bitOffset = 192
			OR_secW_bitOffset = 128
		BREAK 

		CASE 109
		CASE 110
		CASE 113
			OR_primR_bitOffset = 0
			OR_primW_bitOffset = 0
			OR_secR_bitOffset = 384
			OR_secW_bitOffset = 384
		BREAK 

		CASE 111
		CASE 114
			OR_primR_bitOffset = 0
			OR_primW_bitOffset = 0
			OR_secR_bitOffset = 128
			OR_secW_bitOffset = 128
		BREAK 

		CASE 112
			OR_primR_bitOffset = 256
			OR_primW_bitOffset = 256
			OR_secR_bitOffset = 128
			OR_secW_bitOffset = 128
		BREAK 
	ENDS
	
	'Select Fieldbus (Ethernet/IP) channel bit offset
	SWITCH OR_init_fbCh
		CASE 1
			OR_FieldBUS_SB = 161
		BREAK 
		CASE 2
			OR_FieldBUS_SB = 673
		BREAK 
		CASE 3
			OR_FieldBUS_SB = 1185
		BREAK 
		CASE 4
			OR_FieldBUS_SB = 1697
		BREAK 
	ENDS
EndProc


'FUNCTION: Move out gecko pads
'@param OR_Gecko_padOut_inst Gecko device instance: 1 = single or primary, 2 = secondary in dual configuration
'@param OR_Gecko_padOut_wait 0 = don't wait for pad movement, 1 = wait until pad reach the designated position
UsrProc OR_Gecko_padOut(OR_Gecko_padOut_inst AS INTEGER, OR_Gecko_padOut_wait AS INTEGER)
	DIM OR_Gecko_padOut_oW AS INTEGER = 0
	DIM OR_Gecko_padOut_oR AS INTEGER = 0
	
	IF OR_Gecko_padOut_inst = 1
		OR_Gecko_padOut_oW = OR_primW_bitOffset
		OR_Gecko_padOut_oR = OR_primR_bitOffset
	ELSE
		OR_Gecko_padOut_oW = OR_secW_bitOffset
		OR_Gecko_padOut_oR = OR_secR_bitOffset
	ENDIF
	
	SETM O[2 + OR_Gecko_padOut_oW + OR_FieldBUS_SB - 1],  0 'Reset pad in bit
	SETM O[1 + OR_Gecko_padOut_oW + OR_FieldBUS_SB - 1],  1 'Set pad out bit
	STIMER 0.1
	SETM O[1 + OR_Gecko_padOut_oW + OR_FieldBUS_SB - 1],  0 'Reset pad out bit after some time

	IF  OR_Gecko_padOut_wait
		WAITI I[19 + OR_Gecko_padOut_oR + OR_FieldBUS_SB - 1]
	ENDIF

EndProc

'FUNCTION: Move in gecko pads
'@param OR_Gecko_padIn_inst Gecko device instance: 1 = single or primary, 2 = secondary in dual configuration
'@param OR_Gecko_padIn_wait 0 = don't wait for pad movement, 1 = wait until pad reach the designated position
UsrProc OR_Gecko_padIn(OR_Gecko_padIn_inst AS INTEGER, OR_Gecko_padIn_wait AS INTEGER)
	DIM OR_Gecko_padIn_oW AS INTEGER = 0
	DIM OR_Gecko_padIn_oR AS INTEGER = 0
	
	IF OR_Gecko_padIn_inst = 1
		OR_Gecko_padIn_oW = OR_primW_bitOffset
		OR_Gecko_padIn_oR = OR_primR_bitOffset
	ELSE
		OR_Gecko_padIn_oW = OR_secW_bitOffset
		OR_Gecko_padIn_oR = OR_secR_bitOffset
	ENDIF
	
	SETM O[1 + OR_Gecko_padIn_oW + OR_FieldBUS_SB - 1],  0 'Reset pad out bit
	SETM O[2 + OR_Gecko_padIn_oW + OR_FieldBUS_SB - 1],  1 'Set pad in  bit
	STIMER 0.1
	SETM O[2 + OR_Gecko_padIn_oW + OR_FieldBUS_SB - 1],  0 'Reset pad in bit after some time

	IF  OR_Gecko_padIn_wait
		WAITJ I[19 + OR_Gecko_padIn_oR + OR_FieldBUS_SB - 1]
	ENDIF

EndProc

'FUNCTION: Get preload force
'@param OR_Gecko_padIn_inst Gecko device instance: 1 = single or primary, 2 = secondary in dual configuration
'@ret preload force in N
UsrProc OR_Gecko_getF( OR_Gecko_getF_inst AS INTEGER ) AS INTEGER
	DIM OR_Gecko_getF_a AS INTEGER = 0 'address
	DIM OR_Gecko_getF_r AS INTEGER = 0 'return value

	'Set bit address
	OR_Gecko_getF_a = 49

	'Set offset according primary (single) or secondary
	IF OR_Gecko_getF_inst = 1
		OR_Gecko_getF_a = OR_Gecko_getF_a + OR_primR_bitOffset
	ELSE
		OR_Gecko_getF_a = OR_Gecko_getF_a + OR_secR_bitOffset
	ENDIF
	
	'Get data
	CallProc OR_Gecko_getF_r = OR_getI16(OR_Gecko_getF_a)

	OR_Gecko_getF_r = OR_Gecko_getF_r / 100 'get N from 1/100N

RetProc OR_Gecko_getF = OR_Gecko_getF_r
EndProc

'FUNCTION: Get ultrasonic sensor data
'@param OR_Gecko_padIn_inst Gecko device instance: 1 = single or primary, 2 = secondary in dual configuration
'@ret Part distance in mm
UsrProc OR_Gecko_getUS( OR_Gecko_getUS_inst AS INTEGER ) AS INTEGER
	DIM OR_Gecko_getUS_a AS INTEGER = 0 'address
	DIM OR_Gecko_getUS_r AS INTEGER = 0 'return value

	'Set bit address
	OR_Gecko_getUS_a = 65

	'Set offset according primary (single) or secondary
	IF OR_Gecko_getUS_inst = 1
		OR_Gecko_getUS_a = OR_Gecko_getUS_a + OR_primR_bitOffset
	ELSE
		OR_Gecko_getUS_a = OR_Gecko_getUS_a + OR_secR_bitOffset
	ENDIF
	
	'Get data
	CallProc OR_Gecko_getUS_r = OR_getI16(OR_Gecko_getUS_a)

	OR_Gecko_getUS_r = OR_Gecko_getUS_r / 100 'get mm from 1/100mm

RetProc OR_Gecko_getUS = OR_Gecko_getUS_r
EndProc

'FUNCTION: Check, that is device connected or not
'@param OR_Gecko_isConn_inst Gecko device instance: 1 = single or primary, 2 = secondary in dual configuration
'@ret 0 = not connected, 1 = device connected
UsrProc OR_Gecko_isConn( OR_Gecko_isConn_inst AS INTEGER ) AS INTEGER
	DIM OR_Gecko_isConn_a AS INTEGER = 0 'address
	DIM OR_Gecko_isConn_r AS INTEGER = 0 'return value

	'Set bit address
	OR_Gecko_isConn_a = 1

	'Set offset according primary (single) or secondary
	IF OR_Gecko_isConn_inst = 1
		OR_Gecko_isConn_a = OR_Gecko_isConn_a + OR_primR_bitOffset + OR_FieldBUS_SB - 1
	ELSE
		OR_Gecko_isConn_a = OR_Gecko_isConn_a + OR_secR_bitOffset + OR_FieldBUS_SB - 1
	ENDIF
	
	'Get data
	CallProc OR_Gecko_isConn_r = OR_getU16(OR_Gecko_isConn_a)
	
	IF OR_Gecko_isConn_r = 48
		OR_Gecko_isConn_r = 1
	ELSE
		OR_Gecko_isConn_r = 0
	ENDIF

RetProc OR_Gecko_isConn= OR_Gecko_isConn_r
EndProc

'FUNCTION: Check, that is part detected or not
'@param OR_Gecko_padIn_inst Gecko device instance: 1 = single or primary, 2 = secondary in dual configuration
'@ret 0 = part not detected, 1 = part detected
UsrProc OR_Gecko_isPart( OR_Gecko_isPart_inst AS INTEGER ) AS INTEGER
	DIM OR_Gecko_isPart_a AS INTEGER = 0 'address
	DIM OR_Gecko_isPart_r AS INTEGER = 0 'return value

	'Set bit address
	OR_Gecko_isPart_a = 17

	'Set offset according primary (single) or secondary
	IF OR_Gecko_isPart_inst = 1
		OR_Gecko_isPart_a = OR_Gecko_isPart_a + OR_primR_bitOffset
	ELSE
		OR_Gecko_isPart_a = OR_Gecko_isPart_a + OR_secR_bitOffset
	ENDIF
	
	'Get data
	OR_Gecko_isPart_r = I[OR_Gecko_isPart_a + OR_FieldBUS_SB - 1]


RetProc OR_Gecko_isPart= OR_Gecko_isPart_r
EndProc

'FUNCTION: Get pad worn state
'@param OR_Gecko_padIn_inst Gecko device instance: 1 = single or primary, 2 = secondary in dual configuration
'@ret 0 = pads are good (no error), 1 = pads are worn
UsrProc OR_Gecko_padSt( OR_Gecko_padSt_inst AS INTEGER ) AS INTEGER
	DIM OR_Gecko_padSt_a AS INTEGER = 0 'address
	DIM OR_Gecko_padSt_r AS INTEGER = 0 'return value

	'Set bit address
	OR_Gecko_padSt_a = 18

	'Set offset according primary (single) or secondary
	IF OR_Gecko_padSt_inst = 1
		OR_Gecko_padSt_a = OR_Gecko_padSt_a + OR_primR_bitOffset
	ELSE
		OR_Gecko_padSt_a = OR_Gecko_padSt_a + OR_secR_bitOffset
	ENDIF
	
	'Get data
	OR_Gecko_padSt_r = I[OR_Gecko_padSt_a + OR_FieldBUS_SB - 1]


RetProc OR_Gecko_padSt= OR_Gecko_padSt_r
EndProc

'FUNCTION: Get pad position
'@param OR_Gecko_padIn_inst Gecko device instance: 1 = single or primary, 2 = secondary in dual configuration
'@ret 1=pad out, 0=pad in
UsrProc OR_Gecko_padPos( OR_Gecko_padPos_inst AS INTEGER ) AS INTEGER
	DIM OR_Gecko_padPos_a AS INTEGER = 0 'address
	DIM OR_Gecko_padPos_r AS INTEGER = 0 'return value

	'Set bit address
	OR_Gecko_padPos_a = 19

	'Set offset according primary (single) or secondary
	IF OR_Gecko_padPos_inst = 1
		OR_Gecko_padPos_a = OR_Gecko_padPos_a + OR_primR_bitOffset
	ELSE
		OR_Gecko_padPos_a = OR_Gecko_padPos_a + OR_secR_bitOffset
	ENDIF
	
	'Get data
	OR_Gecko_padPos_r = I[OR_Gecko_padPos_a + OR_FieldBUS_SB - 1]

RetProc OR_Gecko_padPos= OR_Gecko_padPos_r
EndProc

'FUNCTION: Get HEX values
'@param OR_HEX_get_f: 	reqested force/torque instance, valid inputs: Fx, Fy, Fz, Tx, Ty, Tz
'				force values are in 1/10N, torque values are in 1/100Nm
'				eg.: Fz = sensor force value on z axis, Tx = sensor torque value on x axis
UsrProc OR_HEX_get(OR_HEX_get_f AS STRING) AS INTEGER
	DIM OR_HEX_get_r AS INTEGER = 0 'return value
	DIM OR_HEX_get_a AS INTEGER = 0 'address
	
	IF OR_HEX_get_f = "Fx"
		OR_HEX_get_a = 65
	ELSEIF OR_HEX_get_f = "Fy"
		OR_HEX_get_a = 81
	ELSEIF OR_HEX_get_f = "Fz"
		OR_HEX_get_a = 97
	ELSEIF OR_HEX_get_f = "Tx"
		OR_HEX_get_a = 113
	ELSEIF OR_HEX_get_f = "Ty"
		OR_HEX_get_a = 129
	ELSEIF OR_HEX_get_f = "Tz"
		OR_HEX_get_a = 145
	ELSE
		L1% = 0
		OR_HEX_get_r = 0
	ENDIF
	
	IF OR_HEX_get_a > 0 
		CallProc OR_HEX_get_r = OR_getI16(OR_HEX_get_a)
	ENDIF

RetProc OR_HEX_get = OR_HEX_get_r
EndProc

'FUNCTION: Zeroing the HEX sensor values (the actual force/torque will be 0)
UsrProc OR_HEX_zero()

	SETM O[2 + OR_FieldBUS_SB - 1], 0 'Reset unzero bit
	SETM O[1 + OR_FieldBUS_SB - 1], 1 'Set zero bit
	STIMER 0.1
	SETM O[1 + OR_FieldBUS_SB - 1], 0 'Reset zero bit

EndProc

'FUNCTION: Unzero the HEX sensor
UsrProc OR_HEX_unzero()

	SETM O[1 + OR_FieldBUS_SB - 1], 0 'Reset zero bit
	SETM O[2 + OR_FieldBUS_SB - 1], 1 'Set unzero bit
	STIMER 0.1
	SETM O[2 + OR_FieldBUS_SB - 1], 0 'Reset unzero bit

EndProc

'FUNCTION: Check that is device connected or not
'@ret 1 = connected, 0 = not connected
UsrProc OR_HEX_isConn() AS INTEGER
	DIM OR_HEX_isConn_a AS INTEGER = 0 'address
	DIM OR_HEX_isConn_r AS INTEGER = 0 'return value

	'Set bit address
	OR_HEX_isConn_a = 1 + OR_FieldBUS_SB -1

	'Get data
	CallProc OR_HEX_isConn_r = OR_getU16(OR_HEX_isConn_a)
	
	IF OR_HEX_isConn_r = 64
		OR_HEX_isConn_r = 1
	ELSE
		OR_HEX_isConn_r = 0
	ENDIF

RetProc OR_HEX_isConn = OR_HEX_isConn_r
EndProc


'FUNCTION: Open/Close gripper to specific dictance (mm) with specific force (N)
'@param OR_RGx_move_width open/close distance (mm)
'@param OR_RGx_move_force force (N)
'@param OR_RGx_move_wait shall we wait for command execution? 1=YES, 0=NO
UsrProc OR_RG2FT_move( OR_RG2FT_move_width AS INTEGER, OR_RG2FT_move_force AS INTEGER, OR_RG2FT_move_wait AS INTEGER )
	DIM OR_RG2FT_move_addr AS INTEGER = 0
	DIM OR_RG2FT_move_val AS INTEGER = 0

	'Set width
	OR_RG2FT_move_addr = 1
	OR_RG2FT_move_val = OR_RG2FT_move_width*10 'Width is sent as 1/10mm 
	CallProc OR_setU16(OR_RG2FT_move_addr, OR_RG2FT_move_val )

	'Set force
	OR_RG2FT_move_addr = 17 
	OR_RG2FT_move_val = OR_RG2FT_move_force*10 'Force is sent as 1/10N
	CallProc OR_setU16(OR_RG2FT_move_addr,  OR_RG2FT_move_val )

	'Start move
	SETM O[34 + OR_FieldBUS_SB - 1],0 'Reset motion stop bit
	SETM O[33 + OR_FieldBUS_SB - 1],1 'Set start motion bit
	STIMER 0.1
	SETM O[33 + OR_FieldBUS_SB - 1],0 'Reset start motion bit

	'Wait for gripper motion if needed
	IF OR_RG2FT_move_wait = 1
		STIMER 0.5
		WAITJ I[449 + OR_FieldBUS_SB - 1]
	ENDIF

EndProc

'FUNCTION: Stop the gripper motion
UsrProc OR_RG2FT_stop()

	SETM O[33 + OR_FieldBUS_SB - 1],0 'Reset start motion bit
	SETM O[34 + OR_FieldBUS_SB - 1],1 'Set motion stop bit
	STIMER 0.1
	SETM O[34 + OR_FieldBUS_SB - 1],0 'Reset motion stop bit

EndProc

'FUNCTION: Set actual value as offset for proximity
UsrProc OR_RG2FT_pOffsAct()

	SETM O[98 + OR_FieldBUS_SB - 1], 0 'Reset custom offset value bit L
	SETM O[130 + OR_FieldBUS_SB - 1], 0 'Reset custom offset value bit R
	
	SETM O[97 + OR_FieldBUS_SB - 1], 1 'Set actual measured value bit L
	SETM O[129 + OR_FieldBUS_SB - 1], 1 'Set actual measured value bit R
	STIMER 0.1
	SETM O[97 + OR_FieldBUS_SB - 1], 0 'Reset actual measured value bit L
	SETM O[129 + OR_FieldBUS_SB - 1], 0 'Reset actual measured value bit R

EndProc

'FUNCTION: Set custom offset for proximity
'@param OR_RGx_pOffsVaL_L left proximity offset (mm)
'@param OR_RGx_pOffsVaL_R right proximity offset (mm)
UsrProc OR_RG2FT_pOffsVaL( OR_RG2FT_pOffsVaL_L AS INTEGER, OR_RG2FT_pOffsVaL_R AS INTEGER)
	DIM OR_RG2FT_pOffsVaL_a AS INTEGER = 0 'address
	DIM OR_RG2FT_pOffsVaL_v AS INTEGER = 0 'value

	'Set left offset value
	OR_RG2FT_pOffsVaL_a = 81
	CallProc OR_setU16(OR_RG2FT_pOffsVaL_a, OR_RG2FT_pOffsVaL_L )

	'Set right offset value
	OR_RG2FT_pOffsVaL_a = 113
	CallProc OR_setU16(OR_RG2FT_pOffsVaL_a,  OR_RG2FT_pOffsVaL_R )

	'Select offset type
	SETM O[97 + OR_FieldBUS_SB - 1],0 'Reset actual measured value bit L
	SETM O[129 + OR_FieldBUS_SB - 1],0 'Reset actual measured value bit R

	SETM O[98 + OR_FieldBUS_SB - 1],1 'Set custom offset value bit L
	SETM O[130 + OR_FieldBUS_SB - 1],1 'Set custom offset value bit R
	STIMER 0.1
	SETM O[98 + OR_FieldBUS_SB - 1],0 'Reset custom offset value bit L
	SETM O[130 + OR_FieldBUS_SB - 1],0 'Reset custom offset value bit R


EndProc

'FUNCTION: 
UsrProc OR_RG2FT_hexZero()

	SETM O[50 + OR_FieldBUS_SB - 1], 0 'Reset unzero bit
	SETM O[49 + OR_FieldBUS_SB - 1], 1 'Set zero bit
	STIMER 0.1
	SETM O[49 + OR_FieldBUS_SB - 1], 0 'Reset zero bit

EndProc

'FUNCTION: 
UsrProc OR_RG2FT_hexUnzero()

	SETM O[49 + OR_FieldBUS_SB - 1], 0 'Reset zero bit
	SETM O[50 + OR_FieldBUS_SB - 1], 1 'Set unzero bit
	STIMER 0.1
	SETM O[50 + OR_FieldBUS_SB - 1], 0 'Reset unzero bit

EndProc

'FUNCTION: Get left proximity value (mm)
UsrProc OR_RG2FT_getLProx() AS INTEGER
	DIM OR_RG2FT_getLProx_r AS INTEGER = 0 'return value
	DIM OR_RG2FT_getLProx_a AS INTEGER = 0 'address
	
	OR_RG2FT_getLProx_a = 369
	CallProc OR_RG2FT_getLProx_r = OR_getI16(OR_RG2FT_getLProx_a)

RetProc OR_RG2FT_getLProx = OR_RG2FT_getLProx_r
EndProc

'FUNCTION: Get right proximity value (mm)
UsrProc OR_RG2FT_getRProx() AS INTEGER
	DIM OR_RG2FT_getRProx_r AS INTEGER = 0 'return value
	DIM OR_RG2FT_getRProx_a AS INTEGER = 0 'address
	
	OR_RG2FT_getRProx_a = 401
	CallProc OR_RG2FT_getRProx_r = OR_getI16(OR_RG2FT_getRProx_a)

RetProc OR_RG2FT_getRProx = OR_RG2FT_getRProx_r
EndProc

'FUNCTION: Get HEX values
'@param OR_RG2FT_getHex_f: 	reqested force/torque instance, valid inputs: lFx, lFy, lFz, lTx, lTy, lTz, rFx, rFy, rFz, rTx, rTy, rTz
'				force values are in 1/10N, torque values are in 1/100Nm
'				eg.: lFz = left sensor force value on z axis, rTx = right sensor torque value on x axis
UsrProc OR_RG2FT_getHex(OR_RG2FT_getHex_f AS STRING) AS INTEGER
	DIM OR_RG2FT_getHex_r AS INTEGER = 0 'return value
	DIM OR_RG2FT_getHex_a AS INTEGER = 0 'address

	IF OR_RG2FT_getHex_f = "lFx"
		OR_RG2FT_getHex_a = 65
	ELSEIF OR_RG2FT_getHex_f = "lFy"
		OR_RG2FT_getHex_a = 81
	ELSEIF OR_RG2FT_getHex_f = "lFz"
		OR_RG2FT_getHex_a = 97
	ELSEIF OR_RG2FT_getHex_f = "lTx"
		OR_RG2FT_getHex_a = 113
	ELSEIF OR_RG2FT_getHex_f = "lTy"
		OR_RG2FT_getHex_a = 129
	ELSEIF OR_RG2FT_getHex_f = "lTz"
		OR_RG2FT_getHex_a = 145
	ELSEIF OR_RG2FT_getHex_f = "rFx"
		OR_RG2FT_getHex_a = 241
	ELSEIF OR_RG2FT_getHex_f = "rFy"
		OR_RG2FT_getHex_a = 257
	ELSEIF OR_RG2FT_getHex_f = "rFz"
		OR_RG2FT_getHex_a = 273
	ELSEIF OR_RG2FT_getHex_f = "rTx"
		OR_RG2FT_getHex_a = 289
	ELSEIF OR_RG2FT_getHex_f = "rTy"
		OR_RG2FT_getHex_a = 305
	ELSEIF OR_RG2FT_getHex_f = "rTz"
		OR_RG2FT_getHex_a = 321
	ELSE
		OR_RG2FT_getHex_a = 0
		OR_RG2FT_getHex_r = 0
	ENDIF

	
	IF OR_RG2FT_getHex_a > 0 
		CallProc OR_RG2FT_getHex_r = OR_getI16(OR_RG2FT_getHex_a)
	ENDIF

RetProc OR_RG2FT_getHex = OR_RG2FT_getHex_r
EndProc

'FUNCTION: Get gripper actual width (mm)
'@ret Actual width in mm
UsrProc OR_RG2FT_getWidth() AS INTEGER
	DIM OR_RG2FT_getWidth_r AS INTEGER = 0 'return value
	DIM OR_RG2FT_getWidth_a AS INTEGER = 0 'address
	
	OR_RG2FT_getWidth_a = 433
	CallProc OR_RG2FT_getWidth_r = OR_getI16(OR_RG2FT_getWidth_a)

	OR_RG2FT_getWidth_r = OR_RG2FT_getWidth_r/10 'to mm

RetProc OR_RG2FT_getWidth = OR_RG2FT_getWidth_r
EndProc

'FUNCTION: Check that is device connected or not
'@ret 1 = connected, 0 = not connected
UsrProc OR_RG2FT_isConn() AS INTEGER
	DIM OR_RG2FT_isConn_a AS INTEGER = 0 'address
	DIM OR_RG2FT_isConn_r AS INTEGER = 0 'return value

	'Set bit address
	OR_RG2FT_isConn_a = 1 + OR_FieldBUS_SB -1

	'Get data
	CallProc OR_RG2FT_isConn_r = OR_getU16(OR_RG2FT_isConn_a)
	
	IF OR_RG2FT_isConn_r = 34
		OR_RG2FT_isConn_r = 1
	ELSE
		OR_RG2FT_isConn_r = 0
	ENDIF

RetProc OR_RG2FT_isConn = OR_RG2FT_isConn_r
EndProc

'FUNCTION: Get gripper status (busy)
'@ret 1 = busy, 0= idle
UsrProc OR_RG2FT_isBusy() AS INTEGER
	DIM OR_RG2FT_isBusy_a AS INTEGER = 0 'address
	DIM OR_RG2FT_isBusy_r AS INTEGER = 0 'return value

	'Set bit address
	OR_RG2FT_isBusy_a = 449 + OR_FieldBUS_SB -1

	'Get bitstatus
	OR_RG2FT_isBusy_r = I[OR_RG2FT_isBusy_a]

RetProc OR_RG2FT_isBusy = OR_RG2FT_isBusy_r
EndProc

'FUNCTION: Check, that is grip detected or not (part gripped)
'@ret 1 = grip detected, 0 = no grip detected
UsrProc OR_RG2FT_isGrip() AS INTEGER
	DIM OR_RG2FT_isGrip_a AS INTEGER = 0 'address
	DIM OR_RG2FT_isGrip_r AS INTEGER = 0 'return value

	'Set bit address
	OR_RG2FT_isGrip_a = 450 + OR_FieldBUS_SB -1

	'Get bitstatus
	OR_RG2FT_isGrip_r = I[OR_RG2FT_isGrip_a]

RetProc OR_RG2FT_isGrip = OR_RG2FT_isGrip_r
EndProc

'FUNCTION: Grip or release with VG10
'@param OR_RGx_grip_inst Gripper instance 1 = primary(or single), 2 = secondary
'@param OR_VG10_grip_vacA Vacuum level of channel A in %
'@param OR_VG10_grip_vacB Vacuum level of channel B in %
'@param OR_VG10_grip_wait Wait until the vacuum builds up: 0=don't wait, 1=wait untilt the vacuum reaches the required level
UsrProc OR_VG10_grip(OR_VG10_grip_inst AS INTEGER, OR_VG10_grip_vacA AS INTEGER, OR_VG10_grip_vacB AS INTEGER, OR_VG10_grip_wait AS INTEGER )
	DIM OR_VG10_grip_addrA AS INTEGER = 0
	DIM OR_VG10_grip_valA AS INTEGER = 0
	DIM OR_VG10_grip_addrB AS INTEGER = 0
	DIM OR_VG10_grip_valB AS INTEGER = 0
	DIM OR_VG10_grip_bitOffs AS INTEGER = 0
	DIM OR_VG10_grip_vA AS INTEGER = 0
	DIM OR_VG10_grip_vB AS INTEGER = 0

	'Set offset according primary (single) or secondary
	IF OR_VG10_grip_inst = 1
		OR_VG10_grip_bitOffs = OR_primW_bitOffset
	ELSE
		OR_VG10_grip_bitOffs = OR_secW_bitOffset
	ENDIF

	'Control byte lowest bit always on (idle) not used
	SETM O[1 + OR_VG10_grip_bitOffs + OR_FieldBUS_SB -1],  1
	SETM O[17 + OR_VG10_grip_bitOffs + OR_FieldBUS_SB -1],  1

	'Set vacuum if needed
	IF OR_VG10_grip_vacA > 0
		OR_VG10_grip_addrA = 33 + OR_VG10_grip_bitOffs
		CallProc OR_setU16(OR_VG10_grip_addrA,  OR_VG10_grip_vacA )
	ENDIF

	IF OR_VG10_grip_vacB > 0
		OR_VG10_grip_addrB = 49 + OR_VG10_grip_bitOffs
		CallProc OR_setU16(OR_VG10_grip_addrB, OR_VG10_grip_vacB )
	ENDIF

	'Control gripper according operation types
	IF OR_VG10_grip_vacA = 0 AND OR_VG10_grip_vacB = 0
		SETM O[2 + OR_VG10_grip_bitOffs + OR_FieldBUS_SB -1],  1
		SETM O[18 + OR_VG10_grip_bitOffs + OR_FieldBUS_SB -1],  1
		OR_VG10_grip_wait = 0 'not need to wait if both channel is released
	ELSEIF OR_VG10_grip_vacA > 0 AND OR_VG10_grip_vacB > 0
		SETM O[2 + OR_VG10_grip_bitOffs + OR_FieldBUS_SB -1],  0
		SETM O[18 + OR_VG10_grip_bitOffs + OR_FieldBUS_SB -1],  0
	ELSEIF OR_VG10_grip_vacA = 0 AND OR_VG10_grip_vacB > 0
		SETM O[2 + OR_VG10_grip_bitOffs + OR_FieldBUS_SB -1],  1
		SETM O[18 + OR_VG10_grip_bitOffs + OR_FieldBUS_SB -1],  0
	ELSE
		SETM O[2 + OR_VG10_grip_bitOffs + OR_FieldBUS_SB -1],  0
		SETM O[18 + OR_VG10_grip_bitOffs + OR_FieldBUS_SB -1],  1
	ENDIF

	'If  we have to wait for the vacuum reach the achived level
	WHILE OR_VG10_grip_wait
		STIMER 0.1
		CallProc OR_VG10_grip_vA = OR_VG10_getVacA(OR_VG10_grip_inst)
		CallProc OR_VG10_grip_vB = OR_VG10_getVacB(OR_VG10_grip_inst)
		IF OR_VG10_grip_vA >= OR_VG10_grip_vacA AND OR_VG10_grip_vB >= OR_VG10_grip_vacB
			OR_VG10_grip_wait = 0
		ENDIF
	ENDW

EndProc

'FUNCTION: Set current limit for VG10
'@param OR_RGx_setCur_inst gripper instance 1 = primary(or single), 2 = secondary
'@param OR_VG10_setCur_mA Current limit in mA, valid range: 100-1000
UsrProc OR_VG10_setCur(OR_VG10_setCur_inst AS INTEGER, OR_VG10_setCur_mA AS INTEGER )
	DIM OR_VG10_setCur_addr AS INTEGER = 0

	OR_VG10_setCur_addr = 65

	'Shift address according primary (single) or secondary
	IF OR_VG10_setCur_inst = 1
		OR_VG10_setCur_addr = OR_VG10_setCur_addr + OR_primW_bitOffset
	ELSE
		OR_VG10_setCur_addr = OR_VG10_setCur_addr + OR_secW_bitOffset
	ENDIF

	'Write register
	CallProc OR_setU16(OR_VG10_setCur_addr, OR_VG10_setCur_mA)

EndProc

'FUNCTION: Check VG10 connection
'@param OR_RGx_isConn_inst gripper instance 1 = primary(or single), 2 = secondary
'@ret VG10 connection: 0=not connected, 1=connected
UsrProc OR_VG10_isConn(OR_VG10_isConn_inst AS INTEGER ) AS INTEGER
	DIM OR_VG10_isConn_addr AS INTEGER = 0
	DIM OR_VG10_isConn_ret AS INTEGER = 0

	'Set bit address
	OR_VG10_isConn_addr = 1

	'Set offset according primary (single) or secondary
	IF OR_VG10_isConn_inst = 1
		OR_VG10_isConn_addr = OR_VG10_isConn_addr + OR_primR_bitOffset
	ELSE
		OR_VG10_isConn_addr = OR_VG10_isConn_addr + OR_secR_bitOffset
	ENDIF

	OR_VG10_isConn_addr = OR_VG10_isConn_addr + OR_FieldBUS_SB -1

	'Get data
	CallProc OR_VG10_isConn_ret = OR_getU16(OR_VG10_isConn_addr)
	
	IF OR_VG10_isConn_ret = 16
		OR_VG10_isConn_ret = 1
	ELSE
		OR_VG10_isConn_ret = 0
	ENDIF

RetProc OR_VG10_isConn = OR_VG10_isConn_ret
EndProc

'FUNCTION: Get actual current limit
'@param OR_RGx_getLim_inst gripper instance 1 = primary(or single), 2 = secondary
'@ret Current limit in mA
UsrProc OR_VG10_getLim( OR_VG10_getLim_inst AS INTEGER ) AS INTEGER
	DIM OR_VG10_getLim_addr AS INTEGER = 0
	DIM OR_VG10_getLim_ret AS INTEGER = 0

	'Set bit address
	OR_VG10_getLim_addr = 17

	'Set offset according primary (single) or secondary
	IF OR_VG10_getLim_inst = 1
		OR_VG10_getLim_addr = OR_VG10_getLim_addr + OR_primR_bitOffset
	ELSE
		OR_VG10_getLim_addr = OR_VG10_getLim_addr + OR_secR_bitOffset
	ENDIF
	
	'Get data
	CallProc OR_VG10_getLim_ret = OR_getU16(OR_VG10_getLim_addr)

RetProc OR_VG10_getLim = OR_VG10_getLim_ret
EndProc

'FUNCTION: Get vacuum level of channel A
'@param OR_RGx_getVacA_inst gripper instance 1 = primary(or single), 2 = secondary
'@ret Vacuum level in %
UsrProc OR_VG10_getVacA( OR_VG10_getVacA_inst AS INTEGER ) AS INTEGER
	DIM OR_VG10_getVacA_addr AS INTEGER = 0
	DIM OR_VG10_getVacA_ret AS INTEGER = 0

	'Set bit address
	OR_VG10_getVacA_addr = 33

	'Set offset according primary (single) or secondary
	IF OR_VG10_getVacA_inst = 1
		OR_VG10_getVacA_addr = OR_VG10_getVacA_addr + OR_primR_bitOffset
	ELSE
		OR_VG10_getVacA_addr = OR_VG10_getVacA_addr + OR_secR_bitOffset
	ENDIF
	
	'Get data
	CallProc OR_VG10_getVacA_ret = OR_getU16(OR_VG10_getVacA_addr)

	OR_VG10_getVacA_ret = OR_VG10_getVacA_ret / 10 'get % from 1/10%

RetProc OR_VG10_getVacA = OR_VG10_getVacA_ret
EndProc

'FUNCTION: Get vacuum level of channel B
'@param OR_RGx_getVacB_inst gripper instance 1 = primary(or single), 2 = secondary
'@ret Vacuum level in %
UsrProc OR_VG10_getVacB( OR_VG10_getVacB_inst AS INTEGER ) AS INTEGER
	DIM OR_VG10_getVacB_addr AS INTEGER = 0
	DIM OR_VG10_getVacB_ret AS INTEGER = 0

	'Set bit address
	OR_VG10_getVacB_addr = 49

	'Set offset according primary (single) or secondary
	IF OR_VG10_getVacB_inst = 1
		OR_VG10_getVacB_addr = OR_VG10_getVacB_addr + OR_primR_bitOffset
	ELSE
		OR_VG10_getVacB_addr = OR_VG10_getVacB_addr + OR_secR_bitOffset
	ENDIF
	
	'Get data
	CallProc OR_VG10_getVacB_ret = OR_getU16(OR_VG10_getVacB_addr)

	OR_VG10_getVacB_ret = OR_VG10_getVacB_ret / 10 'get % from 1/10%

RetProc OR_VG10_getVacB = OR_VG10_getVacB_ret
EndProc


'FUNCTION: Open/Close gripper to specific dictance (mm) with specific force (N)
'@param OR_RGx_move_inst gripper instance 1 = primary(or single), 2 = secondary
'@param OR_RGx_move_width open/close distance (mm)
'@param OR_RGx_move_force force (N)
'@param OR_RGx_move_wait shall we wait for command execution? 1=YES, 0=NO
UsrProc OR_RGx_move(OR_RGx_move_inst AS INTEGER, OR_RGx_move_width AS INTEGER, OR_RGx_move_force AS INTEGER, OR_RGx_move_wait AS INTEGER )
	DIM OR_RGx_move_addr AS INTEGER = 0
	DIM OR_RGx_move_val AS INTEGER = 0
	DIM OR_RGx_move_offsW AS INTEGER = 0
	DIM OR_RGx_move_offsR AS INTEGER = 0

	'Set offset according primary (single) or secondary
	IF OR_RGx_move_inst = 1
		OR_RGx_move_offsW = OR_primW_bitOffset
		OR_RGx_move_offsR =OR_primR_bitOffset
	ELSE
		OR_RGx_move_offsW = OR_secW_bitOffset
		OR_RGx_move_offsR = OR_secR_bitOffset
	ENDIF

	'Set width
	OR_RGx_move_addr = 1 + OR_RGx_move_offsW 
	OR_RGx_move_val = OR_RGx_move_width*10 'Width is sent as 1/10mm 
	CallProc OR_setU16(OR_RGx_move_addr, OR_RGx_move_val )

	'Set force
	OR_RGx_move_addr = 17 + OR_RGx_move_offsW 
	OR_RGx_move_val = OR_RGx_move_force*10 'Force is sent as 1/10N
	CallProc OR_setU16(OR_RGx_move_addr,  OR_RGx_move_val )

	'Start move
	SETM O[34 + OR_RGx_move_offsW + OR_FieldBUS_SB - 1],0 'Reset motion stop bit
	SETM O[33 + OR_RGx_move_offsW + OR_FieldBUS_SB - 1],1 'Set start motion bit
	STIMER 0.1
	SETM O[33 + OR_RGx_move_offsW + OR_FieldBUS_SB - 1],0 'Reset start motion bit

	'Wait for gripper motion if needed
	IF OR_RGx_move_wait = 1
		STIMER 0.5
		WAITJ I[65 + OR_RGx_move_offsR + OR_FieldBUS_SB - 1]
	ENDIF

EndProc

'FUNCTION: Check gripper connection
'@ret Connection state: 0=not connected, 1=connected
UsrProc OR_RGx_isConn(OR_RGx_isConn_inst AS INTEGER ) AS INTEGER
	DIM OR_RGx_isConn_addr AS INTEGER = 0
	DIM OR_RGx_isConn_ret AS INTEGER = 0

	'Set bit address
	OR_RGx_isConn_addr = 1

	'Set offset according primary (single) or secondary
	IF OR_RGx_isConn_inst = 1
		OR_RGx_isConn_addr = OR_RGx_isConn_addr + OR_primR_bitOffset
	ELSE
		OR_RGx_isConn_addr = OR_RGx_isConn_addr + OR_secR_bitOffset
	ENDIF

	OR_RGx_isConn_addr = OR_RGx_isConn_addr + OR_FieldBUS_SB -1
	
	'Get data
	CallProc OR_RGx_isConn_ret = OR_getU16(OR_RGx_isConn_addr)
	
	IF OR_RGx_isConn_ret = 32 OR OR_RGx_isConn_ret = 33
		OR_RGx_isConn_ret = 1
	ELSE
		OR_RGx_isConn_ret = 0
	ENDIF

RetProc OR_RGx_isConn = OR_RGx_isConn_ret
EndProc

'FUNCTION: Check grip detected
'@ret Grip detected: 0=no grip, 1=grip detected
UsrProc OR_RGx_isGrip(OR_RGx_isGrip_inst AS INTEGER ) AS INTEGER
	DIM OR_RGx_isGrip_addr AS INTEGER = 0
	DIM OR_RGx_isGrip_ret AS INTEGER = 0

	'Set bit address
	OR_RGx_isGrip_addr = 66

	'Set offset according primary (single) or secondary
	IF OR_RGx_isGrip_inst = 1
		OR_RGx_isGrip_addr = OR_RGx_isGrip_addr + OR_primR_bitOffset
	ELSE
		OR_RGx_isGrip_addr = OR_RGx_isGrip_addr + OR_secR_bitOffset
	ENDIF

	OR_RGx_isGrip_addr = OR_RGx_isGrip_addr + OR_FieldBUS_SB -1
	
	'Get bit status
	OR_RGx_isGrip_ret = I[OR_RGx_isGrip_addr]

RetProc OR_RGx_isGrip = OR_RGx_isGrip_ret
EndProc


'FUNCTION: Check that is gripper busy or idle
'@param OR_RGx_isBusy_inst Gripper instance: 1=single or primary, 2: secondary in dual configuration
'@ret Gripper state: 0=idle, 1=busy
UsrProc OR_RGx_isBusy(OR_RGx_isBusy_inst AS INTEGER ) AS INTEGER
	DIM OR_RGx_isBusy_addr AS INTEGER = 0
	DIM OR_RGx_isBusy_ret AS INTEGER = 0

	'Set bit address
	OR_RGx_isBusy_addr = 65

	'Set offset according primary (single) or secondary
	IF OR_RGx_isBusy_inst = 1
		OR_RGx_isBusy_addr = OR_RGx_isBusy_addr + OR_primR_bitOffset
	ELSE
		OR_RGx_isBusy_addr = OR_RGx_isBusy_addr + OR_secR_bitOffset
	ENDIF

	OR_RGx_isBusy_addr = OR_RGx_isBusy_addr + OR_FieldBUS_SB -1
	
	'Get bit status
	OR_RGx_isBusy_ret = I[OR_RGx_isBusy_addr]

RetProc OR_RGx_isBusy = OR_RGx_isBusy_ret
EndProc

'FUNCTION: Check safety switch state
'@param OR_RGx_isSSOn_inst Gripper instance: 1=single or primary, 2: secondary in dual configuration
'@ret Safety switch state: 0=none of the safety switches are triggered, 1=safety switch triggered and the gripper is disabled
UsrProc OR_RGx_isSSOn(OR_RGx_isSSOn_inst AS INTEGER ) AS INTEGER
	DIM OR_RGx_isSSOn_addr AS INTEGER = 0
	DIM OR_RGx_isSSOn_offs AS INTEGER = 0
	DIM OR_RGx_isSSOn_ret AS INTEGER = 0


	'Set offset according primary (single) or secondary
	IF OR_RGx_isSSOn_inst = 1
		OR_RGx_isSSOn_offs =  OR_primR_bitOffset
	ELSE
		OR_RGx_isSSOn_offs =  OR_secR_bitOffset
	ENDIF

	'Set bit address
	OR_RGx_isSSOn_addr = 68 + OR_RGx_isSSOn_offs  + OR_FieldBUS_SB -1 'safety circuit 1
	'Get bit status
	OR_RGx_isSSOn_ret = I[OR_RGx_isSSOn_addr]
	
	'If safety circuit 1 is not triggered, check the other one
	IF OR_RGx_isSSOn_ret = 0
		OR_RGx_isSSOn_addr = 70 + OR_RGx_isSSOn_offs  + OR_FieldBUS_SB -1 'safety circuit 2
		
		'Get bit status
		OR_RGx_isSSOn_ret = I[OR_RGx_isSSOn_addr]
	ENDIF

RetProc OR_RGx_isSSOn = OR_RGx_isSSOn_ret
EndProc


'FUNCTION: Set uint16 value to output according the OnRobot regsiter address
'@param OR_setU16_SB starting bit (address) according OnRobot register map
'@param OR_setU16_val uint16 value
UsrProc OR_setU16(OR_setU16_SB As INTEGER, OR_setU16_val AS INTEGER )
	OR_setU16_SB = OR_setU16_SB+OR_FieldBUS_SB-1

	FOR L1%=0 TO 15 STEP 1
		IF(OR_setU16_val mod 2 >= 1)
			SET O[OR_setU16_SB + L1%]
		ELSE
			RESET O[OR_setU16_SB + L1%]
		ENDIF
		OR_setU16_val = OR_setU16_val / 2
	NEXT
EndProc

'FUNCTION: Get uint16 value from input according the OnRobot regsiter address
'@param OR_getU16_SB starting bit (address) according OnRobot register map
UsrProc OR_getU16(OR_getU16_SB AS INTEGER) AS INTEGER
	DIM OR_getU16_ret AS INTEGER = 0

	OR_getU16_SB = (OR_getU16_SB -1 + OR_FieldBUS_SB - 1 )/8 + 4
	OR_getU16_ret  = GETSIGB(OR_getU16_SB,0)+2^8 * GETSIGB(OR_getU16_SB+1,0)

RetProc OR_getU16 = OR_getU16_ret
EndProc

'FUNCTION: Get int16 value from input according the OnRobot regsiter address
'@param OR_getI16_SB starting bit (address) according OnRobot register map
UsrProc OR_getI16(OR_getI16_SB AS INTEGER) AS INTEGER
	DIM OR_getI16_ret AS INTEGER = 0

	OR_getI16_SB = (OR_getI16_SB -1 + OR_FieldBUS_SB - 1 )/8 + 4
	OR_getI16_ret   = GETSIGB(OR_getI16_SB,0)+2^8 * GETSIGB(OR_getI16_SB+1,0)

	IF OR_getI16_ret  >= 32768
		OR_getI16_ret  = OR_getI16_ret - 65536
	ENDIF

RetProc OR_getI16 = OR_getI16_ret 
EndProc
